// PPRAM: Parallel Port Writable RAM */
module PPRAM(CLK, D, PDS, PWE, PINT, PSEL, LSEL, LED);
input CLK;
input [7:0] D;
input PDS;
input PWE;
input PINT;
input PSEL;
input LSEL;
output [7:0] LED;

// Eight registers and an address to remember which to write to next
reg [7:0] mem0, mem1, mem2, mem3, mem4, mem5, mem6, mem7, LED;
reg [2:0] inReg;

// Output selector
reg [2:0] ledReg;

// All the logic is synchronous
always @(posedge CLK) begin
    // PINT Initializes the RAM
    if (PINT) begin
        inReg <= 'h0;
        ledReg <= 'h0;
        mem0 <= 'hA5;
    end
    
    // PDS Enabled when the data is ready
    if (PWE && PDS) begin
        case (inReg) // synopsis full case
          'h0: mem0 <= D;
          'h1: mem1 <= D;
          'h2: mem2 <= D;
          'h3: mem3 <= D;
          'h4: mem4 <= D;
          'h5: mem5 <= D;
          'h6: mem6 <= D;
          'h7: mem7 <= D;
        endcase
    end
    
    // Display the memory location
    case (ledReg) // synopsis full case
      'h0: LED <= mem0;
      'h1: LED <= mem1;
      'h2: LED <= mem2;
      'h3: LED <= mem3;
      'h4: LED <= mem4;
      'h5: LED <= mem5;
      'h6: LED <= mem6;
      'h7: LED <= mem7;
    endcase
    
    // Debugging via PSEL and LSEL
    if (PSEL) LED <= inReg;
    if (LSEL) LED <= ledReg;
end

endmodule 
