module BRAM(CLK, D, ADDR, Write, Value, RST);
input CLK;
input [7:0] D;
input [3:0] ADDR;
input RST;
input RST;

reg [7:0] mem;

// Store the value
always @(posedge CLK) begin
    if (RST) mem <= 'h00;
    else mem <= D;
end

endmodule

// PPRAM: Parallel Port Writable RAM
module PPRAM(CLK, D, PDS, PWE, PINT, PSEL, RST, LSEL, LED);
input CLK;
input [7:0] D;
input PDS;
input PWE;
input PINT;
input PSEL;
input RST;
input LSEL;
output [7:0] LED;

// Eight registers and an address to remember which to write to next
BRAM mem0 (CLK, D, RST);
BRAM mem1 (CLK, D, RST);
reg [2:0] inAddr;

// Either output lights and an address to selector which memory to display
reg [7:0] LED;
reg [2:0] ledAddr;

// Clock based logic
always @(posedge CLK) begin
    // Initializes the RAM
    if (RST | PINT) begin
        inAddr <= 'h0;
        ledAddr <= 'h0;
    end
    else if (PWE & PDS) begin
        mem[0] <= D;
    end

    // Display the memory
    if (LSEL) begin
        LED[7:3] <= 'h0;
        LED[2:0] <= ledAddr;
        ledAddr <= ledAddr + 'h1;
    end
    else if (PSEL) LED <= ledAddr;
    else LED <= mem[0];
end

endmodule
