module NEWADDR(PDATA, CLK, WE, RE, DATA, RESET, ADDR, AW, AR, AWD);

input [7:0] PDATA;
input CLK;
input WE;
input RE;
input DATA;
input RESET;

output [6:0] ADDR;
output AW;
output AR;
output AWD;

reg [6:0] ADDR;
reg AW, AR, AWD;
reg [7:0] length;    // Array length (not including start byte containing this length value)
reg [1:0] state;    // State Machine Control

// Clock based logic
always @(posedge CLK) begin
    // Reset when RESET is high
    if (RESET) begin
        ADDR <= 'h3;
        AW <= 0;
        AR <= 0;
        AWD <= 0;
        state <= 0;
    end
    
    // Reset the state when both enables are low
    if (~WE & ~RE) begin
        ADDR <= 'h3;
        AW <= 0;
        AR <= 0;
        state <= 0;
    end
    // Ready to receive a byte
    else if (WE) begin
        // State machine
        case (state)
          // @(posedge DATA) -- Store and ACK the byte; record the length if first address
          0: if (DATA) begin
              AW <= 1;
              if (ADDR == 'h3) begin
                  length <= PDATA;    // Determine Array length
              end
                  state <= 3;
             end
          
          // Un-ACK and Increment the address
          1: begin
              AW <= 0;
              if (ADDR == length + 3) begin
                  ADDR <= 'h3;
                  AWD <= 1;    // Set Address Write Done (signals to PROCESSARY that data is completely available for processing)
              end
              else begin
                  ADDR <= ADDR + 'h1;
              end
              state <= 0;
             end
          
          // @(negedge DATA) -- The byte has been written
          2: if (~DATA) begin
              AW <= 0;
              state <= 1;
             end
          
          // One cycle to store the data
          3: begin
              AW <= 1;
              state <= 2;
             end
          
        endcase
    end
    // Ready to send a byte
    else if (RE) begin
        // State machine
        case (state)
          // @(posedge DATA) -- Send the byte
          0: if (DATA) begin
              AR <= 1;
              state <= 3;
             end
          
          // Un-ACK and Increment the address
          1: begin
              AR <= 0;
              if (ADDR == length + 3) begin
                  ADDR <= 'h3;
                  AWD <= 0;
              end
              else begin
                  ADDR <= ADDR + 'h1;
              end
              state <= 0;
             end
          
          // @(negedge DATA) -- The byte has been read
          2: if (~DATA) begin
              AR <= 0;
              state <= 1;
             end
             
          // One cycle to read the data
          3: begin
              AR <= 1;
              state <= 2;
             end
          
        endcase
    end
end

endmodule
