module PROCESSARY (DIN, AIN, CLK, START, DOUT, AOUT, BUS_REQ, WR_EN, WR_CLK) ;

input [31:0] DIN ;
input [6:0] AIN ;
input CLK ;
input START;

output [31:0] DOUT ;
output [6:0] AOUT ;
output BUS_REQ ;
output WR_EN ;
output WR_CLK ;

// add your declarations here

reg [31:0] DOUT ;
reg [6:0] AOUT ;
reg BUS_REQ ;
reg WR_EN ;
reg WR_CLK ;
reg [7:0] length;  // Array length (not including start byte containing this length value)
reg [14:0] sum;
reg [7:0] average;
reg [2:0] state1;    // Main Processing State Machine Control
reg [1:0] state2;    // RAM Operations State Machine Control

// add your code here

// Clock based logic
always @(posedge CLK) begin
    if (~START) begin
        // Retain Status-Quo
        BUS_REQ <= 0;
        WR_EN <= 0;
        WR_CLK <= 0;
        state1 <= 0;
    end
    else begin
        // Start Main State Machine Processing
        case (state1)
          0: begin
              BUS_REQ <= 1;    // Seize Control of Address and Data Buses
              AOUT <= 'h0;
              WR_EN <= 0;
              WR_CLK <= 0;
              sum <= 'h3;
              average <= 'h3;
              state1 <= 1;
             end
            
          1: begin
              length <= DIN[31:24];    // Determine Array length
              state1 <= 3;
              state2 <= 0;
             end
       
             
          3: begin
            // Process / Write New Data to RAM
            if (AOUT < (length + 4)) begin
                case (state2)
                  0: begin
                      WR_EN <= 0;
	              WR_CLK <= 0;
                      AOUT <= AOUT + 'h4;
                      state2 <= 1;
                     end
                     
                  1: begin
                      // Calculate Normalized Data for writing to RAM

                      DOUT[7:0] <= length;
                      DOUT[31:8] <= 'haabbcc;
			
                      WR_EN <= 1;    // Write Enable to Enable RAM for Writing
                      state2 <= 2;
                     end
                     
                  2: begin
                      WR_CLK <= 1;    // Write Clock to Write Data
                      state2 <= 3;
                     end
                     
                  3: begin
                      // One cycle to store the data
                      state2 <= 0;
                     end
                     
                endcase
            end
            else begin
                state1 <= 4;
            end
             end
             
          4: begin
              // Holding State on Completion until START Reset by System Reads
              BUS_REQ <= 0;    // Release Buses
	      WR_EN <= 0;
	      WR_CLK <= 0;
	     end    
	        
        endcase
    end
end

endmodule
